// Include standard libraries.
#include <string>
#include <iostream>

// Include socket libraries.
#include "TcpSocket.h"

class SockTest{
	public:
		SockTest();
		~SockTest();
		void run(std::string ipOrHost);
};

/**
 * This is the standard constructor.
 * It does pretty nothing.
 */
SockTest::SockTest() {
	std::cout << "Starting simulation ..." << std::endl;
}

/**
 * This is the standard destructor.
 * It does pretty nothing.
 */
SockTest::~SockTest() {
	std::cout << "Simulation finished ..." << std::endl;
}

/**
 * The run function holds the test program,
 * which is used to test the ray tracer.
 */
void SockTest::run(std::string ipOrHost) {
	try {
		// Loop over all standard ports.
		for(int port = 0; port < 1024; port++) {
			// Instance the TcpSocket object, ...
			TcpSocket socket = TcpSocket();
		
			// ... and try to connect to each port of the given host.
			if( socket.connectTo(ipOrHost, port) ) {
				// Print out port.
				std::cout << port << " (" << socket.getService() << ") open" << std::endl;
			}
		}
	}
	
	catch(TcpSocketException e) {
		std::cerr << e << std::endl;
	}
}

/**
 * The main function starts the simulation.
 */
int main(int argc, char *argv[]) {
	// Check if enough parameters given.
	if( argc >= 2 ) {
		// Instance the test class ...
		SockTest simulation = SockTest();
	
		// ... and start the simulation.
		simulation.run(argv[1]);
	}
	
	else {
		std::cerr << "To few parameters given."
			<< "Usage: " << argv[0] << " <ip or host>" << std::endl;
		return 1;
	}
	
	return 0;
}
