#ifndef TCP_SOCKET_CPP
#define TCP_SOCKET_CPP

// Include header.
#include "TcpSocket.h"

// Define some constants.
#define BUFFER_SIZE 4096

TcpSocket::TcpSocket() throw(TcpSocketException) {
	// Initialize a tcp socket.
	sock = socket(PF_INET, SOCK_STREAM, 0);

	// Validate socket ...
	if( sock == -1 ) {
		// ... and handle errors.
		throw TcpSocketException("Failed to initialize socket.");
	}
}

TcpSocket::~TcpSocket() {
	// Close socket.
	close(this->sock);    
}

bool TcpSocket::connectTo(std::string host, int port) throw(TcpSocketException) {
	// Check if given host address is an ip.
	if( inet_aton(host.c_str(), &addr.sin_addr) == 0 ) {
		// If not get data by hostname ...
		hostData = gethostbyname(host.c_str());
		
		// Validate host data ...
		if (!hostData) {
			// ... and handle errors.
			throw TcpSocketException("Failed to resolve " + host + ".");
		}
		
		// Finally set host address.
		addr.sin_addr = *(struct in_addr*) hostData->h_addr;
	}
	
	else {
		// If ip was given set host address by ip.
		addr.sin_addr.s_addr = inet_addr(host.c_str());
	}

	// Init address structure.
	addr.sin_port = htons(port);
	addr.sin_family = AF_INET;

	// Establish connection to given ip address and port ...
	if( connect(this->sock, (struct sockaddr*) &addr, sizeof(addr)) == -1 ) {
		// ... and handle errors.
			throw TcpSocketException("Failed to connect to " + host + ".");
	}

	return true;
}

bool TcpSocket::write(std::string data) throw(TcpSocketException) {
	// Send data and store size of transfered data. 
	int bytes = send(sock, data.c_str(), strlen(data.c_str()), 0);
	
	// Check if anything was send ...
	if (bytes == -1) {
		// ... and handle errors.
		throw TcpSocketException("Failed to send data.");
	}

	return true;
}

std::string TcpSocket::read() throw(TcpSocketException) {
	// Initialize buffer ...
	char buffer[BUFFER_SIZE];

	// Read data and store size of received data.
	int bytes = recv(sock, buffer, sizeof(buffer) - 1, 0);
	
	// Check if anything was received ...
	if (bytes == -1) {
		throw TcpSocketException("Failed to read data.");
	}

	// Declare result std::string ...
	std::string result;
	
	// ... and append received data.
	result.assign(buffer, bytes * sizeof(char));

	return result;
}

std::string TcpSocket::getService() {
	// Get the service name by the current port.
	struct servent *serv = getservbyport(addr.sin_port, "tcp");
	
	// Check if name was found.
	if (serv) {
		return serv->s_name;
	}
	
	return "unknown";
}

#endif
